﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading.Tasks;
using ParallelUtil;

namespace TestParallelUtils
{
    /// <summary>
    /// Summary description for TestAggregateExceptionHandling
    /// </summary>
    [TestClass]
    public class TestAggregateExceptionHandling
    {
        public TestAggregateExceptionHandling()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void TestCatchAggregate()
        {
            Task badTask = Task.Factory.StartNew(() =>
                {
                    throw new InvalidOperationException("Bad operation");
                });

            try
            {
                badTask.Wait();
                Assert.Fail("Should not get here...");
            }
            catch (AggregateException error)
            {
                error
                    .Catch<InvalidOperationException>(e =>
                    {
                        Assert.AreSame(e.GetType(), typeof(InvalidOperationException));
                        Assert.AreEqual(e.Message, "Bad operation");
                    })
                    .ThrowUnHandled();

            }
        }

        public void TestThrowUnHandledAggregate()
        {
            Task badTask = Task.Factory.StartNew(() =>
            {
                throw new InvalidOperationException("Bad operation");
            });

            try
            {
                try
                {
                    badTask.Wait();
                    Assert.Fail("Should not get here...");
                }
                catch (AggregateException error)
                {
                    error
                        .Catch<ArgumentException>(e =>
                        {
                            Assert.Fail();
                        })
                        .ThrowUnHandled();

                }

                Assert.Fail();
            }
            catch (AggregateException aggregateError)
            {
                Assert.IsTrue(aggregateError.InnerExceptions.Count == 1);
                Assert.IsTrue(aggregateError.InnerExceptions[0].GetType() == typeof(InvalidOperationException));
                Assert.AreEqual(aggregateError.InnerExceptions[0].Message, "Bad operation");
            }
        }

        [TestMethod]
        public void TestManyExceptions()
        {
            Task[] tasks = new Task[2];

            tasks[0] = Task.Factory.StartNew(() =>
            {
                throw new InvalidOperationException("Bad operation");
            });

            tasks[1] = Task.Factory.StartNew(() =>
            {
                throw new ArgumentException("Bad Argument");
            });


            try
            {
                Task.WaitAll(tasks);
                Assert.Fail("Should not get here...");
            }
            catch (AggregateException error)
            {
                error
                    .Catch<ArgumentException>(e =>
                    {
                        Assert.AreEqual(e.GetType(), typeof(ArgumentException));
                        Assert.AreEqual(e.Message, "Bad Argument");
                    })
                    .Catch<InvalidOperationException>(e =>
                        {
                            Assert.AreEqual(e.GetType(), typeof(InvalidOperationException));
                            Assert.AreEqual(e.Message, "Bad operation");
                        })
                    .ThrowUnHandled();

            }



        }

        [TestMethod]
        public void TestReplaceAndThrow()
        {
            Task badTask = Task.Factory.StartNew(() =>
            {
                throw new InvalidOperationException("Bad operation");
            });

            try
            {
                try
                {
                    badTask.Wait();
                    Assert.Fail("Should not get here...");
                }
                catch (AggregateException error)
                {
                    error
                        .Catch<InvalidOperationException>(e =>
                        {
                            return new Exception("Boom");
                        })
                        .ThrowUnHandled();

                }
            }
            catch (AggregateException error)
            {
                Assert.AreEqual(1, error.InnerExceptions.Count());
                Assert.AreSame(typeof(Exception), error.InnerExceptions[0].GetType());
                Assert.AreEqual("Boom", error.InnerExceptions[0].Message);
            }
        }
    }
}
